@@ -1,11 +1,15 @@
 package Mail::Milter::Authentication::Handler::SMIME;
 use strict;
 use warnings;
+use Mail::Milter::Authentication 2;
 use base 'Mail::Milter::Authentication::Handler';
-use version; our $VERSION = version->declare('v1.1.7');
-
+our $VERSION = '2.20180318'; # VERSION
+# ABSTRACT: Authentication Milter Module for validation of SMIME
 use English qw{ -no_match_vars };
 use Sys::Syslog qw{:standard :macros};
+use Mail::AuthenticationResults::Header::Entry;
+use Mail::AuthenticationResults::Header::SubEntry;
+use Mail::AuthenticationResults::Header::Comment;
 
 use Convert::X509;
 use Crypt::SMIME;
@@ -74,24 +78,21 @@ sub eom_callback {
 
         if ( $self->{'found'} == 0 ) {
             if ( !( $config->{'hide_none'} ) ) {
-                $self->add_auth_header(
-                    $self->format_header_entry( 'smime', 'none' ),
-                );
+                my $header = Mail::AuthenticationResults::Header::Entry->new()->set_key( 'smime' )->set_value( 'none' );
+                $self->add_auth_header( $header );
             }
             $self->{'metric_result'} = 'none';
         }
         elsif ( $self->{'added'} == 0 ) {
-            $self->add_auth_header(
-                $self->format_header_entry( 'smime', 'temperror' ),
-            );
+            my $header = Mail::AuthenticationResults::Header::Entry->new()->set_key( 'smime' )->set_value( 'temperror' );
+            $self->add_auth_header( $header );
             $self->{'metric_result'} = 'error';
         }
     };
     if ( my $error = $@ ) {
         $self->log_error( 'SMIME Execution Error ' . $error );
-        $self->add_auth_header(
-            $self->format_header_entry( 'smime', 'temperror' ),
-        );
+        my $header = Mail::AuthenticationResults::Header::Entry->new()->set_key( 'smime' )->set_value( 'temperror' );
+        $self->add_auth_header( $header );
         $self->{'metric_result'} = 'error';
     }
 
@@ -209,9 +210,8 @@ sub _check_mime {
             $self->_decode_certs( 'fail', $signatures, $all_certs, $part_id );
             ## ToDo extract the reason for failure and add as header comment
             if ( $self->{'added'} == 0 ) {
-                $self->add_auth_header(
-                    $self->format_header_entry( 'smime', 'fail' ),
-                );
+                my $header = Mail::AuthenticationResults::Header::Entry->new()->set_key( 'smime' )->set_value( 'fail' );
+                $self->add_auth_header( $header );
                 $self->{'metric_result'} = 'fail';
                 $self->{'added'} = 1;
             }
@@ -231,7 +231,6 @@ sub _decode_certs {
 
     my $seen = {};
 
-
     SIGNATURE:
     foreach my $cert ( @{$signatures} ) {
 
@@ -252,22 +251,24 @@ sub _decode_certs {
         next SIGNATURE if $seen->{ $serial };
         $seen->{ $serial } = 1;
 
-        my @results;
-        push @results, $self->format_header_entry( 'body.smime-identifier', $subject->{'E'}[0] )
-            . '(' . $self->format_header_comment( $subject->{'CN'}[0] ) . ')';
-        push @results, $self->format_header_entry( 'body.smime-part', $part_id );
-        push @results, $self->format_header_entry( 'body.smime-serial', $serial );
+        my $header = Mail::AuthenticationResults::Header::Entry->new()->set_key( 'smime' )->safe_set_value( $passfail );
+
+        my $header_id = Mail::AuthenticationResults::Header::SubEntry->new()->set_key( 'body.smime-identifier' )->safe_set_value( $subject->{'E'}[0] );
+        $header_id->add_child( Mail::AuthenticationResults::Header::Comment->new()->safe_set_value( $subject->{'CN'}[0] ) );
+        $header->add_child( $header_id );
+
+        $header->add_child( Mail::AuthenticationResults::Header::SubEntry->new()->set_key( 'body.smime-part' )->safe_set_value( $part_id ) );
+        $header->add_child( Mail::AuthenticationResults::Header::SubEntry->new()->set_key( 'body.smime-serial' )->safe_set_value( $serial ) );
+
         my $issuer_text = join( ',', map{ $_ . '=' . $issuer->{$_}[0] } sort keys (%{$issuer}) );
         $issuer_text =~ s/\"/ /g;
-        push @results, 'body.smime-issuer="' . $self->format_ctext( $issuer_text ) . '"' ;
-        push @results, 'x-smime-valid-from="' . $self->format_ctext( $from ) . '"';
-        push @results, 'x-smime-valid-to="'   . $self->format_ctext( $to ) . '"';
-        $self->add_auth_header(
-            join( "\n        ",
-                $self->format_header_entry( 'smime', $passfail ),
-                @results,
-            )
-        );
+
+        $header->add_child( Mail::AuthenticationResults::Header::SubEntry->new()->set_key( 'body.smime-issuer' )->safe_set_value( $issuer_text ) );
+        $header->add_child( Mail::AuthenticationResults::Header::SubEntry->new()->set_key( 'x-smime-valid-from' )->safe_set_value( $from ) );
+        $header->add_child( Mail::AuthenticationResults::Header::SubEntry->new()->set_key( 'x-smime-valid-to' )->safe_set_value( $to ) );
+
+        $self->add_auth_header($header);
+
         $self->{'metric_result'} = $passfail;
         $self->{'added'} = 1;
     }
@@ -292,23 +293,24 @@ sub _decode_certs {
         next CERT if $seen->{ $serial };
         $seen->{ $serial } = 1;
 
-        my @results;
-        push @results, $self->format_header_entry( 'body.smime-part', $part_id );
-        push @results, $self->format_header_entry( 'x-smime-chain-identifier', ( $subject->{'E'}[0] || 'null' ) )
-            . ' (' . $self->format_header_comment( $subject->{'CN'}[0] ) . ')';
-        push @results, $self->format_header_entry( 'x-smime-chain-serial', $serial );
+        my $header = Mail::AuthenticationResults::Header::Entry->new()->set_key( 'x-smime-chain' )->safe_set_value( 'info' );
+
+        $header->add_child( Mail::AuthenticationResults::Header::SubEntry->new()->set_key( 'body.smime-part' )->safe_set_value( $part_id ) );
+        my $chain_id_value = $subject->{'E'}[0];
+        my $chain_id_comment = $subject->{'CN'}[0];
+        $chain_id_value = 'null' if ! defined $chain_id_value;
+        $chain_id_comment = 'null' if ! defined $chain_id_comment;
+        my $chain_id = Mail::AuthenticationResults::Header::SubEntry->new()->set_key( 'x-smime-chain-identifier' )->safe_set_value( $chain_id_value ); 
+        $chain_id->add_child( Mail::AuthenticationResults::Header::Comment->new()->safe_set_value( $chain_id_comment ) );
+        $header->add_child( $chain_id );
+        $header->add_child( Mail::AuthenticationResults::Header::SubEntry->new()->set_key( 'x-smime-chain-serial' )->safe_set_value( $serial ) );
         my $issuer_text = join( ',', map{ $_ . '=' . $issuer->{$_}[0] } sort keys (%{$issuer}) );
         $issuer_text =~ s/\"/ /g;
-        push @results, 'x-smime-chain-issuer="' . $self->format_ctext( $issuer_text ) . '"' ;
-        push @results, 'x-smime-chain-valid-from="' . $self->format_ctext( $from ) . '"';
-        push @results, 'x-smime-chain-valid-to="'   . $self->format_ctext( $to ) . '"';
-                $self->add_auth_header(
-                    join( "\n        ",
-                        $self->format_header_entry( 'x-smime-chain', 'info' ),
-                        @results,
-                    )
-                );
-                $self->{'added'} = 1;
+        $header->add_child( Mail::AuthenticationResults::Header::SubEntry->new()->set_key( 'x-smime-chain-issuer' )->safe_set_value( $issuer_text ) );
+        $header->add_child( Mail::AuthenticationResults::Header::SubEntry->new()->set_key( 'x-smime-chain-valid-from' )->safe_set_value( $from ) );
+        $header->add_child( Mail::AuthenticationResults::Header::SubEntry->new()->set_key( 'x-smime-chain-valid-to' )->safe_set_value( $to ) );
+        $self->add_auth_header( $header );
+        $self->{'added'} = 1;
     }
 
     return;
@@ -318,9 +320,17 @@ sub _decode_certs {
 
 __END__
 
+=pod
+
+=encoding UTF-8
+
 =head1 NAME
 
-  Authentication Milter - SMIME Module
+Mail::Milter::Authentication::Handler::SMIME - Authentication Milter Module for validation of SMIME
+
+=head1 VERSION
+
+version 2.20180318
 
 =head1 DESCRIPTION
 
@@ -333,8 +343,6 @@ Check SMIME signed email for validity.
             "pki_store" : "/etc/ssl/certs"
         },
 
-=head1 SYNOPSIS
-
 =head2 CONFIG
 
 Add a block to the handlers section of your config as follows.
@@ -344,16 +352,15 @@ Add a block to the handlers section of your config as follows.
             "pki_store"         : "/etc/ssl/certs"      | The location of your trusted root certs
         },
 
-=head1 AUTHORS
+=head1 AUTHOR
 
-Marc Bradshaw E<lt>marc@marcbradshaw.netE<gt>
+Marc Bradshaw <marc@marcbradshaw.net>
 
-=head1 COPYRIGHT
-
-Copyright 2017
-
-This library is free software; you may redistribute it and/or
-modify it under the same terms as Perl itself.
+=head1 COPYRIGHT AND LICENSE
 
+This software is copyright (c) 2018 by Marc Bradshaw.
 
+This is free software; you can redistribute it and/or modify it under
+the same terms as the Perl 5 programming language system itself.
 
+=cut
